@@ -6,20 +6,25 @@ log = get_logger(__name__)
 
 
 class _Collector(object):
-    def __init__(self, selection, token):
+    def __init__(self, completion_context: ICompletionContext, token):
         from robotframework_ls.impl.string_matcher import RobotStringMatcher
         from robotframework_ls.impl.string_matcher import (
             build_matchers_with_resource_or_library_scope,
         )
+        from robotframework_ls.robot_config import create_convert_keyword_format_func
 
         token_str = token.value
 
-        self.completion_items = []
-        self.selection = selection
+        self.completion_items: List[dict] = []
+        self.completion_context = completion_context
+        self.selection = completion_context.sel
         self.token = token
 
         self._matcher = RobotStringMatcher(token_str)
         self._scope_matchers = build_matchers_with_resource_or_library_scope(token_str)
+        config = completion_context.config
+
+        self._convert_keyword_format = create_convert_keyword_format_func(config)
 
     def accepts(self, keyword_name):
         if self._matcher.accepts_keyword_name(keyword_name):
@@ -43,8 +48,13 @@ class _Collector(object):
         from robotframework_ls.impl.protocols import IKeywordArg
 
         label = keyword_found.keyword_name
-        text = label
 
+        if keyword_found.library_name:
+            # If we found the keyword in a library, convert its format depending on
+            # the user configuration.
+            label = self._convert_keyword_format(label)
+
+        text = label
         arg: IKeywordArg
         for i, arg in enumerate(keyword_found.keyword_args):
             if arg.is_keyword_arg or arg.is_star_arg or arg.default_value is not None:
@@ -65,7 +75,7 @@ class _Collector(object):
 
         # text_edit = None
         return CompletionItem(
-            keyword_found.keyword_name,
+            label,
             kind=keyword_found.completion_item_kind,
             text_edit=text_edit,
             insertText=text_edit.newText,
@@ -104,7 +114,7 @@ def complete(completion_context: ICompletionContext) -> List[dict]:
     if token_info is not None:
         token = ast_utils.get_keyword_name_token(token_info.node, token_info.token)
         if token is not None:
-            collector = _Collector(completion_context.sel, token)
+            collector = _Collector(completion_context, token)
             collect_keywords(completion_context, collector)
 
             return collector.completion_items
