@@ -54,12 +54,12 @@ class Sentinel(Enum):
 
 def check_implements(x: T) -> T:
     """
-    Helper to check if a class implements some protocol. 
+    Helper to check if a class implements some protocol.
 
     :important: It must be the last method in a class due to
                 https://github.com/python/mypy/issues/9266
 
-	Example:
+        Example:
 
     def __typecheckself__(self) -> None:
         _: IExpectedProtocol = check_implements(self)
@@ -104,10 +104,10 @@ class IEndPoint(Protocol):
     def notify(self, method: str, params: Any = None):
         """Send a JSON RPC notification to the client.
 
-         Args:
-             method (str): The method name of the notification to send
-             params (any): The payload of the notification
-         """
+        Args:
+            method (str): The method name of the notification to send
+            params (any): The payload of the notification
+        """
 
     def request(self, method: str, params=None) -> IFuture:
         """Send a JSON RPC request to the client.
@@ -156,11 +156,30 @@ class IRequestCancellable(Protocol):
         """
 
 
+class IRequestHandler(Protocol):
+    def __call__(self, request_name: str, msg_id: Any, params: Any) -> bool:
+        """
+        :param request_name:
+            The name of the request to be handled.
+
+        :param msg_id:
+            The id of the message (to which a response should be generated).
+
+        :param params:
+            The parameters received in the request.
+
+        :return:
+            True if the request was handled (in which case, if multiple request
+            handlers are registered, others aren't processed anymore) and False
+            otherwise.
+        """
+
+
 class ILanguageServerClientBase(IRequestCancellable, Protocol):
     def request_async(self, contents: Dict) -> Optional[IIdMessageMatcher]:
         """
         API which allows to wait for the message to complete.
-        
+
         To use:
             message_matcher = client.request_async(contents)
             if message_matcher is not None:
@@ -169,11 +188,11 @@ class ILanguageServerClientBase(IRequestCancellable, Protocol):
                     msg = message_matcher.msg
                 else:
                     # Timed out
-            
+
         :param contents:
             Something as:
             {"jsonrpc": "2.0", "id": msg_id, "method": method_name, "params": params}
-            
+
         :return _MessageMatcher:
         """
 
@@ -189,7 +208,7 @@ class ILanguageServerClientBase(IRequestCancellable, Protocol):
         :return:
             The returned message if everything goes ok.
             `default` if the communication dropped in the meanwhile and timeout was None.
-        
+
         :raises:
             TimeoutError if the timeout was given and no answer was given at the available time
             (including if the communication was dropped).
@@ -200,7 +219,7 @@ class ILanguageServerClientBase(IRequestCancellable, Protocol):
     ) -> IMessageMatcher:
         """
         Can be used as:
-        
+
         message_matcher = language_server.obtain_pattern_message_matcher(
             {"method": "textDocument/publishDiagnostics"}
         )
@@ -209,6 +228,9 @@ class ILanguageServerClientBase(IRequestCancellable, Protocol):
     def obtain_id_message_matcher(self, message_id) -> IMessageMatcher:
         pass
 
+    def register_request_handler(self, message: str, handler: IRequestHandler) -> None:
+        pass
+
     def write(self, contents):
         pass
 
@@ -328,7 +350,7 @@ class ILanguageServerClient(ILanguageServerClientBase, Protocol):
     def settings(self, settings: Dict):
         """
         :param settings:
-            Something as: 
+            Something as:
             {"settings": {"robot": {"pythonpath": [case4_path]}}}
         """
 
@@ -427,13 +449,13 @@ class IConfig(Protocol):
         """
         :param key:
             The setting to be gotten (i.e.: my.setting.to.get)
-            
+
         :param expected_type:
             The type which we're expecting.
-            
+
         :param default:
             If given, return this value instead of throwing a KeyError.
-            
+
         :raises:
             KeyError if the setting could not be found and default was not provided.
         """
@@ -445,7 +467,17 @@ class IConfig(Protocol):
         pass
 
     def set_override_settings(self, override_settings: dict) -> None:
-        pass
+        """
+        Used to override settings with the keys given (note: any existing
+        override setting will be removed and all the keys here will be set to
+        override the initial settings -- use update_override_settings to keep
+        other existing overrides).v
+        """
+
+    def update_override_settings(self, override_settings: dict) -> None:
+        """
+        Used to update existing override settings with the keys given.
+        """
 
     def set_workspace_dir(self, workspace: str) -> None:
         pass
@@ -485,8 +517,7 @@ class ILanguageServer(IConfigProvider):
 
 
 class IDirCache(Protocol):
-    """
-    """
+    """ """
 
     def store(self, key: Any, value: Any) -> None:
         """
@@ -494,8 +525,8 @@ class IDirCache(Protocol):
 
         :param key:
             The key to be persisted. It's repr(key) is used to calculate
-            the key filename on the disk. 
-            
+            the key filename on the disk.
+
         :note that the values do a round-trip with json (so, caveats
         such as saving a tuple and loading a list apply).
         """
@@ -503,10 +534,10 @@ class IDirCache(Protocol):
     def load(self, key: Any, expected_class: Type) -> Any:
         """
         Loads a previously persisted value.
-        
-        If it doesn't exist, there's some error loading or the expected 
-        class doesn't match the loaded value a KeyError is thrown. 
-        
+
+        If it doesn't exist, there's some error loading or the expected
+        class doesn't match the loaded value a KeyError is thrown.
+
         :note: users should check that the cache value is what's expected when it's
            gotten (as the data may become corrupted on disk or may change across
            versions).
@@ -686,3 +717,19 @@ class ActionResult(Generic[T]):
 class EnvEntry(TypedDict):
     key: str
     value: str
+
+
+class LibraryVersionDict(TypedDict):
+    library: str
+    version: str
+
+
+class LibraryVersionInfoDict(TypedDict):
+    success: bool
+
+    # if success == False, this can be some message to show to the user
+    message: Optional[str]
+
+    # Note that if the library was found but the version doesn't match, the
+    # result should still be provided.
+    result: Optional[LibraryVersionDict]
